/* screen.c
 * Part of the RTAudio.Client project.
 * This file contains the ncurses function calls that make the client
 * look all nice and pretty.
 * Written by Electronic One <dkaye@cyberspc.mb.ca>
 */

#include "curses.h"
#include <unistd.h>

#include "exit.h"

/* This little but very important sub-routine initializes all colour pairs
 * accessible over normal ANSI code (64 colour pairs).
 */
void init_all_colour_pairs(void)
{
   short back=0, fore=0, pair=0;  /* pair ought to be init'ed at 0 */
   
   for(back=0;  back < 8;  ++back)
     {
	for(fore=0;  fore < 8;  ++fore)
	  {
	     /* This will get an error when doing color pair 0,
	      * but I don't care.
	      */
	     init_pair(pair, fore, back);
	     ++pair;
	  }
     }
   
}

int get_color(int f, int b)
{
   if(
      (f >= COLOR_BLACK) &&
      (f <= COLOR_WHITE) &&
      (b >= COLOR_BLACK) &&
      (b <= COLOR_WHITE)
     )
     return ((b * 8) + f) << 8;
   else
     return -1;
}

void startup_ncurses(void)
{
   initscr();
   start_color();
   cbreak(); 
   noecho(); 
   nonl(); 
   intrflush(stdscr, FALSE);
   keypad(stdscr, TRUE);
   init_all_colour_pairs();
   curs_set(0);
}

chtype rgb2bgr(chtype c)
{
   chtype hi, lo, mid;
   
   /* Extract bits */
   hi = c & 4;  /* 00000100b */
   mid= c & 2;  /* 00000010b */
   lo = c & 1;  /* 00000001b */

   /* Shift bits */
   lo <<= 2;
   hi >>= 2;
   
   return hi + mid + lo;
}

/* This highly specialized function converts a character and attribute into 
 * a chtype type which contains both of those things.
 */
chtype TheDraw_to_chtype(unsigned char c, unsigned char a)
{
   chtype fore, back, blink, bright, ci;
   chtype ch;
   
   /* Extract character attributes */
   fore   =   7 & a;  /* 00000111b & a */
   back   = 112 & a;  /* 01110000b & a */
   blink  = 128 & a;  /* 10000000b & a */
   bright =   8 & a;  /* 00001000b & a */
   
   back >>= 4;  /* Move it into the lowest bit position so that it reflects
		* colour number (ie: COLOR_GREEN=2)
		*/
   
   /* Compensate for the incompatibility between ncurses and IBM */
   back = rgb2bgr(back);
   fore = rgb2bgr(fore);
   
   ch = c;  /* Assign the character */
   
   /* Now calculate the colour pair which holds these attributes */
   ci = ((back * 8) + fore);
   ch |= ci << 8;

   /* Handle the bright attribute */
   ch |= (bright) ? A_BOLD : A_NORMAL;

   /* Handle the blink attribute */
   ch |= (blink) ? A_BLINK : A_NORMAL;
   
   return ch;
}


/* This is a quick little hack which loads a TheDraw compatible .C file */
void load_screen(unsigned char *scr)
{
   int x, y;
   
   move(0, 0);
   
   for(y=0; y < 25; ++y)
     {
	for(x=0; x < 80; ++x)
	  {
	     stdscr->_line[y].text[x] = 
	       TheDraw_to_chtype(*(scr + (y * 160) + (x * 2)), 
				 *(scr + (y * 160) + (x * 2) + 1));
	  }
     }
   
   touchwin(stdscr);
   refresh();
}

void load_main_screen(void)
{
   int i;
   
   bkgdset(A_BOLD|get_color(COLOR_WHITE, COLOR_BLUE));
   attrset(A_BOLD|get_color(COLOR_WHITE, COLOR_BLUE));
   mvprintw(0, 0, "  Input  Output  Mode  Settings  Windows                                  Help  ");
   bkgdset(A_BOLD|get_color(COLOR_YELLOW, COLOR_BLUE));
   attrset(A_BOLD|get_color(COLOR_YELLOW, COLOR_BLUE));
   mvaddch(0, 2, 'I');
   mvaddch(0, 9, 'O');
   mvaddch(0, 17, 'M');
   mvaddch(0, 23, 'S');
   mvaddch(0, 33, 'W');
   mvaddch(0, 74, 'H');
   bkgdset(get_color(COLOR_RED, COLOR_BLACK));
   attrset(get_color(COLOR_RED, COLOR_BLACK));
   for(i=1;  i <= stdscr->_maxy; ++i)
     mvaddstr(i, 0, "");
   refresh();
}



